# -*- coding: utf-8 -*-
import time
from flask import current_app as app, _app_ctx_stack

def _send_to_statsd(
        resource, bucket, stat_type='counter', value=None):
    if ((app.config['STATSD_MONITORED_DOMAINS'] and 
        resource not in app.config['STATSD_MONITORED_DOMAINS']) or 
        app.debug):
        # are not interested in this resource
        return
    
    if stat_type == 'timer':
        if value:
            app.statsd.timing(bucket, value)
    else:
        app.statsd.incr(bucket)
    

def _start_timer():
    top = _app_ctx_stack.top
    if not hasattr(top, 'statsd_timer'):
        top.statsd_timer = time.time()

def _end_timer_get_delta():
    start = None
    top = _app_ctx_stack.top
    if hasattr(top, 'statsd_timer'):
        start = top.statsd_timer
        del top.statsd_timer
    if start:
        return int((time.time() - start) * 1000)
    return None

def pre_GET(resource, request, lookup):
    # montoring get requests
    for bucket in (resource + '.counter.all', resource + '.counter.get'):
        _send_to_statsd(resource, bucket)
    # start timer here
    _start_timer()

def pre_HEAD(resource, request, lookup):
    # montoring get requests
    for bucket in (resource + '.counter.all', resource + '.counter.head'):
        _send_to_statsd(resource, bucket)
    # start timer here
    _start_timer()

def pre_POST(resource, request):
    # montoring get requests
    for bucket in (resource + '.counter.all', resource + '.counter.post'):
        _send_to_statsd(resource, bucket)
    # start timer here
    _start_timer()

def pre_PATCH(resource, request, lookup):
    # montoring get requests
    for bucket in (resource  + '.counter.all', resource + '.counter.patch'):
        _send_to_statsd(resource, bucket)
    # start timer here
    _start_timer()

def pre_PUT(resource, request, lookup):
    # montoring get requests
    for bucket in (resource, resource + '.counter.put'):
        _send_to_statsd(resource + '.counter.all', bucket)
    # start timer here
    _start_timer()

def pre_DELETE(resource, request, lookup):
    # montoring get requests
    for bucket in (resource, resource + '.counter.delete'):
        _send_to_statsd(resource + '.counter.all', bucket)
    # start timer here
    _start_timer()


def post_GET(resource, request, payload):
    # logging timers on post
    _send_to_statsd(
        resource, resource + '.timing.get', value=_end_timer_get_delta(),
        stat_type='timer'
    )

def post_HEAD(resource, request, payload):
    # logging timers on post
    _send_to_statsd(
        resource, resource + '.timing.head', value=_end_timer_get_delta(), 
        stat_type='timer'
    )

def post_POST(resource, request, payload):
    # logging timers on post
    _send_to_statsd(
        resource, resource + '.timing.post', value=_end_timer_get_delta(),
        stat_type='timer'
    )
    
def post_PATCH(resource, request, payload):
    # logging timers on post
    _send_to_statsd(
        resource, resource + '.timing.patch', value=_end_timer_get_delta(),
        stat_type='timer'
    )
    
def post_PUT(resource, request, payload):
    # logging timers on post
    _send_to_statsd(
        resource, resource + '.timing.put', value=_end_timer_get_delta(),
        stat_type='timer'
    )
    
def post_DELETE(resource, request, payload):
    # logging timers on post
    _send_to_statsd(
        resource,
        resource + '.timing.delete', value=_end_timer_get_delta(),
        stat_type='timer'
    )



# -*- coding: utf-8 -*-

import unittest
from eve import Eve
from flask import g
from eve_statsd import StatsD
from statsdmock import StatsdMockServer

settings = {
    'DOMAIN': {
        'foo': {
            'schema': {
                'name': {},
            },
            'resource_methods': ['POST', 'GET'],
        },
    },
}


class TestBase(unittest.TestCase):
    def setUp(self):
        # set up eve
        self.app = Eve(settings=settings)
        self.app.config['TESTING'] = True
        StatsD(self.app)
        self.test_client = self.app.test_client()
        
        # set up statsd mock
        self.server = StatsdMockServer()
        self.server.start()
        
    
    def tearDown(self):
        # Hook
        self.server.stop()
        del self.server
        super(TestBase, self).tearDown()

    def test_counter(self):
        r = self.test_client.get('/foo')
        
        # let's check statsd mock
        self.assertEqual(r.status_code, 200) 
        self.server.wait('foo.counter.get', 1)
        self.server.wait('foo.counter.all', 1)

        data = list(self.server.metrics['foo.counter.get'])
        self.assertEqual(len(data), 1)

        self.assertEqual(data[0]['value'], '1')
        self.assertEqual(data[0]['type'], 'counter')
        
        data = list(self.server.metrics['foo.counter.all'])
        self.assertEqual(len(data), 1)

        self.assertEqual(data[0]['value'], '1')
        self.assertEqual(data[0]['type'], 'counter')
        
        self.test_client.get('/foo')
        self.server.wait('foo.counter.get', 2)
        data = list(self.server.metrics['foo.counter.get'])
        self.assertEqual(len(data), 2)
        
    def test_timer(self):
        r = self.test_client.get('/foo')
        
        # let's check statsd mock
        self.assertEqual(r.status_code, 200) 
        self.server.wait('foo.timing.get', 1)

        data = list(self.server.metrics['foo.timing.get'])
        self.assertEqual(len(data), 1)

        self.assertEqual(data[0]['type'], 'timer')

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
from statsd import StatsClient
from flask import current_app
import hooks

_METHODS = ['GET', 'HEAD', 'POST', 'PATCH', 'PUT', 'DELETE' ]

class StatsD(object):
    
    def __init__(self, app):
        """ Implements the Flask extension pattern.
        """
        self.client = None
        if app is not None:
            self.app = app
            self.init_app(self.app)
        else:
            self.app = None


    def init_app(self, app):
        app.config.setdefault('STATSD_HOST', 'localhost')
        app.config.setdefault('STATSD_PORT', 8125)
        app.config.setdefault('STATSD_PREFIX', None)
        app.config.setdefault('STATSD_MONITORED_DOMAINS', [])
        # app.config.setdefault('STATSD_MONITORED_METHODS', [])

        self.app = app

        self.client = StatsClient(
            host=app.config['STATSD_HOST'],
            port=app.config['STATSD_PORT'],
            prefix=app.config['STATSD_PREFIX']
        )
        self.app.statsd = self
        if not hasattr(app, 'extensions'):
            app.extensions = {}
        app.extensions.setdefault('statsd', {})
        app.extensions['statsd'] = self
        
        self._init_request_hooks()
        self._init_database_hooks()
        self._init_exception_hooks()
        
        
    
    def __getattr__(self, name):
        return getattr(self.client, name, None)

    def _init_request_hooks(self):
        """ initialize pre request hooks"""
        for method_type in ('pre', 'post'):
            for method in _METHODS:
                event = getattr(self.app, 'on_' + method_type + '_' + method)
                event_hook = getattr(hooks, method_type + '_' + method)
                event += event_hook
    
    def _init_database_hooks(self):
        """ initialize database hooks we might want to monitor database calls"""
        pass


    
    def _init_exception_hooks(self):
        # TODO Error handling stats
        return
        # registers exception counters
        #self.app.register_error_handler(Exception, hooks.exception_hook)
    
        



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name='Eve-Statsd',
    version='0.0.1',
    description='Statsd integration for eve',
    long_description=(
        open('README.rst').read() + '\n\n' + open('CHANGELOG.rst').read()),
    license=open('LICENSE.txt').read(),
    author='Salton Massally',
    author_email='salton.massally@gmail.com',
    url='https://github.com/tarzan0820/eve-statsd',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['eve', 'statsd'],
    test_suite='nose.collector',
    tests_require=['nose', 'statsdmock'],
    keywords = ['monitoring', 'logging', 'eve', 'statsd'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
